@@ -1,12 +1,62 @@
-/* des.c */
-/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au).
+/* lib/des/des.c */
+/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au)
  * All rights reserved.
+ * 
+ * This file is part of an SSL implementation written
+ * by Eric Young (eay@mincom.oz.au).
+ * The implementation was written so as to conform with Netscapes SSL
+ * specification.  This library and applications are
+ * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
+ * as long as the following conditions are aheared to.
+ * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.
- * See the COPYRIGHT file in the libdes distribution for more details.
+ * the code are not to be removed.  If this code is used in a product,
+ * Eric Young should be given attribution as the author of the parts used.
+ * This can be in the form of a textual message at program startup or
+ * in documentation (online or textual) provided with the package.
+ * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ * 
+ * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ * 
+ * The licence and distribution terms for any publically available version or
+ * derivative of this code cannot be changed.  i.e. this code cannot simply be
+ * copied and put under another distribution licence
+ * [including the GNU Public Licence.]
  */
 
 #include <stdio.h>
+#include <stdlib.h>
+#ifndef MSDOS
+#include <unistd.h>
+#else
+#include <io.h>
+#define RAND
+#endif
+
+#include <time.h>
+#include "version.h"
+
 #ifdef VMS
 #include <types.h>
 #include <stat.h>
@@ -16,33 +66,26 @@
 #endif
 #include <sys/stat.h>
 #endif
-#if !defined(_LIBC) || defined(NOCONST)
+#if defined(NOCONST)
 #define const
 #endif
 #include "des.h"
-#include "version.h" /* get version definition */
-
 
 #if defined(__STDC__) || defined(VMS) || defined(M_XENIX) || defined(MSDOS)
 #include <string.h>
 #endif
 
-#define bcopy(f,t,n)	memcpy(t,f,(size_t)(n))
-#define bzero(s,n)	memset(s,0,(size_t)(n))
-#define bcmp(a,b,n)	memcmp(a, b,(size_t)(n))
-#define index(s,c)	strchr(s,c)
-
 #ifdef RAND
-#define random() rand()
+#define random rand
 #define srandom(s) srand(s)
 #endif
 
 #ifdef PROTO
 void usage(void);
-int doencryption(void);
-int uufwrite(char *data, int size, int num, FILE *fp);
+void doencryption(void);
+int uufwrite(unsigned char *data, int size, unsigned int num, FILE *fp);
 void uufwriteEnd(FILE *fp);
-int uufread(char *out,int size,int num,FILE *fp);
+int uufread(unsigned char *out,int size,unsigned int num,FILE *fp);
 int uuencode(unsigned char *in,int num,unsigned char *out);
 int uudecode(unsigned char *in,int num,unsigned char *out);
 #else
@@ -67,31 +110,31 @@ int uudecode();
 #define KEYSIZB 1024 /* should hit tty line limit first :-) */
 char key[KEYSIZB+1];
 int do_encrypt,longk=0;
-char *in=NULL,*out=NULL;
 FILE *DES_IN,*DES_OUT,*CKSUM_OUT;
 char uuname[200];
-char uubuf[50];
+unsigned char uubuf[50];
 int uubufnum;
 #define INUUBUFN	(45*100)
 #define OUTUUBUF	(65*100)
-char b[OUTUUBUF];
-char bb[300];
+unsigned char b[OUTUUBUF];
+unsigned char bb[300];
 des_cblock cksum={0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
 char cksumname[200]="";
 
 int vflag,cflag,eflag,dflag,kflag,bflag,fflag,sflag,uflag,flag3,hflag,error;
 
-int main(argc,argv)
+int main(argc, argv)
 int argc;
-char *argv[];
+char **argv;
 	{
 	int i;
 	struct stat ins,outs;
 	char *p;
+	char *in=NULL,*out=NULL;
 
 	vflag=cflag=eflag=dflag=kflag=hflag=bflag=fflag=sflag=uflag=flag3=0;
 	error=0;
-	bzero(key,sizeof(key));
+	memset(key,0,sizeof(key));
 
 	for (i=1; i<argc; i++)
 		{
@@ -203,13 +246,17 @@ char *argv[];
 		{
 		if (vflag) 
 			{
-			fprintf(stderr,"des(1) built with %s\n",version);
+#ifndef _Windows			
+			fprintf(stderr,"des(1) built with %s\n",DES_version);
+#endif			
 			EXIT(1);
 			}
 		else usage();
 		}
 
-	if (vflag) fprintf(stderr,"des(1) built with %s\n",version);
+#ifndef _Windows			
+	if (vflag) fprintf(stderr,"des(1) built with %s\n",DES_version);
+#endif			
 	if (	(in != NULL) &&
 		(out != NULL) &&
 #ifndef MSDOS
@@ -272,7 +319,7 @@ char *argv[];
 void usage()
 	{
 	char **u;
-	static const char *usage[]={
+	static const char *Usage[]={
 "des <options> [input-file [output-file]]",
 "options:",
 "-v         : des(1) version number",
@@ -298,7 +345,7 @@ void usage()
 "             encryption.  Default is tripple cbc, -b makes it tripple ecb.",
 NULL
 };
-	for (u=(char **)usage; *u; u++)
+	for (u=(char **)Usage; *u; u++)
 		{
 		fputs(*u,stderr);
 		fputc('\n',stderr);
@@ -379,7 +426,7 @@ void doencryption()
 				k2[i-8]=k;
 			}
 		des_set_key((C_Block *)k2,ks2);
-		bzero(k2,sizeof(k2));
+		memset(k2,0,sizeof(k2));
 		}
 	else if (longk || flag3)
 		{
@@ -387,7 +434,7 @@ void doencryption()
 			{
 			des_string_to_2keys(key,(C_Block *)kk,(C_Block *)k2);
 			des_set_key((C_Block *)k2,ks2);
-			bzero(k2,sizeof(k2));
+			memset(k2,0,sizeof(k2));
 			}
 		else
 			des_string_to_key(key,(C_Block *)kk);
@@ -409,11 +456,11 @@ void doencryption()
 			}
 
 	des_set_key((C_Block *)kk,ks);
-	bzero(key,sizeof(key));
-	bzero(kk,sizeof(kk));
+	memset(key,0,sizeof(key));
+	memset(kk,0,sizeof(kk));
 	/* woops - A bug that does not showup under unix :-( */
-	bzero(iv,sizeof(iv));
-	bzero(iv2,sizeof(iv2));
+	memset(iv,0,sizeof(iv));
+	memset(iv2,0,sizeof(iv2));
 
 	l=1;
 	rem=0;
@@ -436,7 +483,7 @@ void doencryption()
 			len=l-rem;
 			if (feof(DES_IN))
 				{
-				srandom(time(NULL));
+				srandom((unsigned int)time(NULL));
 				for (i=7-rem; i>0; i--)
 					buf[l++]=random()&0xff;
 				buf[l++]=rem;
@@ -465,7 +512,7 @@ void doencryption()
 						ks,do_encrypt);
 			else if (flag3 && bflag)
 				for (i=0; i<l; i+=8)
-					des_3ecb_encrypt(
+					des_2ecb_encrypt(
 						(des_cblock *)&(buf[i]),
 						(des_cblock *)&(obuf[i]),
 						ks,ks2,do_encrypt);
@@ -473,29 +520,33 @@ void doencryption()
 				{
 				char tmpbuf[8];
 
-				if (rem) bcopy(&(buf[l]),tmpbuf,rem);
+				if (rem) memcpy(tmpbuf,&(buf[l]),
+					(unsigned int)rem);
 				des_3cbc_encrypt(
 					(des_cblock *)buf,(des_cblock *)obuf,
 					(long)l,ks,ks2,(des_cblock *)iv,
 					(des_cblock *)iv2,do_encrypt);
-				if (rem) bcopy(tmpbuf,&(buf[l]),rem);
+				if (rem) memcpy(&(buf[l]),tmpbuf,
+					(unsigned int)rem);
 				}
 			else
 				{
 				des_cbc_encrypt(
 					(des_cblock *)buf,(des_cblock *)obuf,
 					(long)l,ks,(des_cblock *)iv,do_encrypt);
-				if (l >= 8) bcopy(&(obuf[l-8]),iv,8);
+				if (l >= 8) memcpy(iv,&(obuf[l-8]),8);
 				}
-			if (rem) bcopy(&(buf[l]),buf,rem);
+			if (rem) memcpy(buf,&(buf[l]),(unsigned int)rem);
 
 			i=0;
 			while (i < l)
 				{
 				if (uflag)
-					j=uufwrite(obuf,1,l-i,DES_OUT);
+					j=uufwrite(obuf,1,(unsigned int)l-i,
+						DES_OUT);
 				else
-					j=fwrite(obuf,1,l-i,DES_OUT);
+					j=fwrite(obuf,1,(unsigned int)l-i,
+						DES_OUT);
 				if (j == -1)
 					{
 					perror("Write error");
@@ -540,7 +591,7 @@ void doencryption()
 						ks,do_encrypt);
 			else if (flag3 && bflag)
 				for (i=0; i<l; i+=8)
-					des_3ecb_encrypt(
+					des_2ecb_encrypt(
 						(des_cblock *)&(buf[i]),
 						(des_cblock *)&(obuf[i]),
 						ks,ks2,do_encrypt);
@@ -556,7 +607,7 @@ void doencryption()
 				des_cbc_encrypt(
 					(des_cblock *)buf,(des_cblock *)obuf,
 				 	(long)l,ks,(des_cblock *)iv,do_encrypt);
-				if (l >= 8) bcopy(&(buf[l-8]),iv,8);
+				if (l >= 8) memcpy(iv,&(buf[l-8]),8);
 				}
 
 			if (uflag)
@@ -586,7 +637,7 @@ void doencryption()
 				(C_Block *)cksum);
 			while (i != l)
 				{
-				j=fwrite(obuf,1,l-i,DES_OUT);
+				j=fwrite(obuf,1,(unsigned int)l-i,DES_OUT);
 				if (j == -1)
 					{
 					perror("Write error");
@@ -616,26 +667,30 @@ void doencryption()
 		if (l) fclose(CKSUM_OUT);
 		}
 problems:
-	bzero(buf,sizeof(buf));
-	bzero(obuf,sizeof(obuf));
-	bzero(ks,sizeof(ks));
-	bzero(ks2,sizeof(ks2));
-	bzero(iv,sizeof(iv));
-	bzero(iv2,sizeof(iv2));
-	bzero(kk,sizeof(kk));
-	bzero(k2,sizeof(k2));
-	bzero(uubuf,sizeof(uubuf));
-	bzero(b,sizeof(b));
-	bzero(bb,sizeof(bb));
-	bzero(cksum,sizeof(cksum));
+	memset(buf,0,sizeof(buf));
+	memset(obuf,0,sizeof(obuf));
+	memset(ks,0,sizeof(ks));
+	memset(ks2,0,sizeof(ks2));
+	memset(iv,0,sizeof(iv));
+	memset(iv2,0,sizeof(iv2));
+	memset(kk,0,sizeof(kk));
+	memset(k2,0,sizeof(k2));
+	memset(uubuf,0,sizeof(uubuf));
+	memset(b,0,sizeof(b));
+	memset(bb,0,sizeof(bb));
+	memset(cksum,0,sizeof(cksum));
 	if (Exit) EXIT(Exit);
 	}
 
-int uufwrite(data,size,num,fp)
-char *data;
-int size; /* We ignore this parameter but it should be > ~50 I believe */
-int num;
+int uufwrite(data, size, num, fp)
+unsigned char *data;
+int size;
+unsigned int num;
 FILE *fp;
+      
+     /* We ignore this parameter but it should be > ~50 I believe */
+   
+    
 	{
 	int i,j,left,rem,ret=num;
 	static int start=1;
@@ -651,16 +706,16 @@ FILE *fp;
 		{
 		if (uubufnum+num < 45)
 			{
-			bcopy(data,&(uubuf[uubufnum]),num);
+			memcpy(&(uubuf[uubufnum]),data,(unsigned int)num);
 			uubufnum+=num;
 			return(num);
 			}
 		else
 			{
 			i=45-uubufnum;
-			bcopy(data,&(uubuf[uubufnum]),i);
-			j=uuencode(uubuf,45,b);
-			fwrite(b,1,j,fp);
+			memcpy(&(uubuf[uubufnum]),data,(unsigned int)i);
+			j=uuencode((unsigned char *)uubuf,45,b);
+			fwrite(b,1,(unsigned int)j,fp);
 			uubufnum=0;
 			data+=i;
 			num-=i;
@@ -670,19 +725,19 @@ FILE *fp;
 	for (i=0; i<(num-INUUBUFN); i+=INUUBUFN)
 		{
 		j=uuencode(&(data[i]),INUUBUFN,b);
-		fwrite(b,1,j,fp);
+		fwrite(b,1,(unsigned int)j,fp);
 		}
 	rem=(num-i)%45;
 	left=(num-i-rem);
 	if (left)
 		{
 		j=uuencode(&(data[i]),left,b);
-		fwrite(b,1,j,fp);
+		fwrite(b,1,(unsigned int)j,fp);
 		i+=left;
 		}
 	if (i != num)
 		{
-		bcopy(&(data[i]),uubuf,rem);
+		memcpy(uubuf,&(data[i]),(unsigned int)rem);
 		uubufnum=rem;
 		}
 	return(ret);
@@ -700,15 +755,15 @@ FILE *fp;
 		uubuf[uubufnum+1]='\0';
 		uubuf[uubufnum+2]='\0';
 		j=uuencode(uubuf,uubufnum,b);
-		fwrite(b,1,j,fp);
+		fwrite(b,1,(unsigned int)j,fp);
 		}
 	fwrite(end,1,strlen(end),fp);
 	}
 
-int uufread(out,size,num,fp)
-char *out;
+int uufread(out, size, num, fp)
+unsigned char *out;
 int size; /* should always be > ~ 60; I actually ignore this parameter :-) */
-int num;
+unsigned int num;
 FILE *fp;
 	{
 	int i,j,tot;
@@ -721,13 +776,13 @@ FILE *fp;
 		for (;;)
 			{
 			b[0]='\0';
-			fgets(b,300,fp);
+			fgets((char *)b,300,fp);
 			if (b[0] == '\0')
 				{
 				fprintf(stderr,"no 'begin' found in uuencoded input\n");
 				return(-1);
 				}
-			if (strncmp(b,"begin ",6) == 0) break;
+			if (strncmp((char *)b,"begin ",6) == 0) break;
 			}
 		start=0;
 		}
@@ -735,22 +790,22 @@ FILE *fp;
 	tot=0;
 	if (valid)
 		{
-		bcopy(bb,out,valid);
+		memcpy(out,bb,(unsigned int)valid);
 		tot=valid;
 		valid=0;
 		}
 	for (;;)
 		{
 		b[0]='\0';
-		fgets(b,300,fp);
+		fgets((char *)b,300,fp);
 		if (b[0] == '\0') break;
-		i=strlen(b);
+		i=strlen((char *)b);
 		if ((b[0] == 'e') && (b[1] == 'n') && (b[2] == 'd'))
 			{
 			done=1;
 			while (!feof(fp))
 				{
-				fgets(b,300,fp);
+				fgets((char *)b,300,fp);
 				}
 			break;
 			}
@@ -760,13 +815,13 @@ FILE *fp;
 			{
 			/* num to copy to make it a multiple of 8 */
 			j=(num/8*8)-tot-8;
-			bcopy(bb,&(out[tot]),j);
+			memcpy(&(out[tot]),bb,(unsigned int)j);
 			tot+=j;
-			bcopy(&(bb[j]),bb,i-j);
+			memcpy(bb,&(bb[j]),(unsigned int)i-j);
 			valid=i-j;
 			break;
 			}
-		bcopy(bb,&(out[tot]),i);
+		memcpy(&(out[tot]),bb,(unsigned int)i);
 		tot+=i;
 		}
 	return(tot);
@@ -777,11 +832,11 @@ FILE *fp;
 		 	 l|=((unsigned long)(*((c)++))))
 
 #define l2ccc(l,c)      (*((c)++)=(unsigned char)(((l)>>16)&0xff), \
-                         *((c)++)=(unsigned char)(((l)>> 8)&0xff), \
-                         *((c)++)=(unsigned char)(((l)    )&0xff))
+                    *((c)++)=(unsigned char)(((l)>> 8)&0xff), \
+                    *((c)++)=(unsigned char)(((l)    )&0xff))
 
 
-int uuencode(in,num,out)
+int uuencode(in, num, out)
 unsigned char *in;
 int num;
 unsigned char *out;
@@ -814,16 +869,16 @@ unsigned char *out;
 	return(tot);
 	}
 
-int uudecode(in,num,out)
+int uudecode(in, num, out)
 unsigned char *in;
 int num;
 unsigned char *out;
 	{
 	int j,i,k;
-	unsigned int n,space=0;
+	unsigned int n=0,space=0;
 	unsigned long l;
 	unsigned long w,x,y,z;
-	unsigned int blank='\n'-' ';
+	unsigned int blank=(unsigned int)'\n'-' ';
 
 	for (j=0; j<num; )
 		{
